//  2012 SAP AG. All rights reserved.
// Author : SAP.

/**
 * 
 * Global Variable Declaration 
 */

var performanceLogFile;
var performanceLogFileEntry;
var performanceLogFileWriter;
var performanceLog;
var performanceLogsPool=[ ];
var boolPerformanceLogProcessing = false;

/**
 * 	Function to set the Log level for the Client Side to any of the following
 * 
 * 	jQuery.sap.log.Level.FATAL Fatal level(0).
 *	jQuery.sap.log.Level.ERROR Error level(1).
 *	jQuery.sap.log.Level.WARNING Warning level(2).	
 *	jQuery.sap.log.Level.INFO Info level(3).
 *	jQuery.sap.log.Level.DEBUG Debug level(4).
 *	jQuery.sap.log.Level.TRACE Trace level(5).
 * 	jQuery.sap.log.Level.NONE Do not log anything
 *
 *  Higher numeric values represent less important levels
 * 
 *  Setting it to jQuery.sap.log.Level.DEBUG for development purposes
 * 
 * 	@param {jQuery.sap.log.Level} logLevel
 */


function setPerformanceLogLevel(logLevel) 
{
	
	if(logLevel == "ON")
		logLevel = jQuery.sap.log.Level.INFO;
		
	else if(logLevel == "OFF")
		logLevel = jQuery.sap.log.Level.NONE;
    			
	//Set Log Level for Performance Log
	jQuery.sap.log.setLevel(logLevel, "Performance");

}

/**
 * Creates a new info-level entry in the log with the given message, details and calling component.
 *
 * @param {} message Message text to display
 * @param {} detail Details about the message, might be omitted
 * @param {} component Name of the component that produced the log entry
 */
function logPerformanceInfo(message,detail)
{
	var oldLogCount = jQuery.sap.log.getLogEntries().length;

	jQuery.sap.log.info(message,detail,"Performance");
	
	var newLogCount = jQuery.sap.log.getLogEntries().length;
	
	if(oldLogCount!=newLogCount)
	{
		//Check if Performance log file is created. Log the statements only when file is created. 
		if(performanceLogFileEntry)
		{
			addLogToPerformanceLogPool(jQuery.sap.log.getLogEntries()[jQuery.sap.log.getLogEntries().length-1]);
		}
	}	
}


/**
 * Success CallBack Function for createPerformanceLogFile
 * 
 * @param {} performanceFile - Techincal Log File created
 * 
 */
function gotPerformanceLogFileEntry(performanceLogFileEntryReceived)
{
	//console.log("Performance Log File Entry created at"+performanceLogFileEntryReceived.fullPath);
	performanceLogFileEntry = performanceLogFileEntryReceived;
	performanceLogFileEntry.file(gotPerformanceLogFile,failPerformanceLogFile);
}

function gotPerformanceLogFile(performanceLogFileReceived)
{
	//Creating the PerformanceLogFile Writer
	performanceLogFile = performanceLogFileReceived;
	performanceLogFileEntry.createWriter(gotPerformanceLogFileWriter, failPerformanceLogFileWriter);	   
}

function failPerformanceLogFile(error)
{
	//console.log("Failed to create Performance log file. Error is "+error.code+"Launching the app now");
  	
  	if(!app)
			appCreate();
}

/**
 * Success Callback Function for performanceLogFileReceived.createWriter
 * 
 * @param {} performanceLogFileWriterReceived - The received PerformanceLogFile writer
 */

function gotPerformanceLogFileWriter(performanceLogFileWriterReceived)
{
    //console.log("Performance Log File Writer Created");
	
    performanceLogFileWriter = performanceLogFileWriterReceived;
    
    performanceLogFileWriter.onwritestart = function(evt)  
 	{
	  		boolPerformanceLogProcessing = true;
  	};

  	performanceLogFileWriter.onwriteend = function(evt) 
 	{
			processPerformanceLogPool("callback");
 	};     		
  
 	performanceLogFileWriter.onerror = function(evt)
	{
     		//console.log("Performance Log File. Write to file couldnot be performed"+evt.code);
	};

   	for(var i=0;i<listOfLogFilesNames.length;i++)
	{
		if(listOfLogFilesNames[i]=="TraceLog")
		{
				//console.log("Setting the TraceLog file level to "+listOfLogFilesLevels[i]);

				setTraceLogLevel(listOfLogFilesLevels[i]);
				
				if (device.platform.match("Android"))
				{
    				//console.log("Creating Trace Log file for Android "+listOfLogFiles[i]);
    				fileSystem.root.getFile("SAPCustomerLoyaltyActivity/"+listOfLogFiles[i], {create: true}, gotTraceLogFileEntry, failTraceLogFileEntry);
				}
				
				else
				{
    				//console.log("Creating Trace Log file for iOS "+listOfLogFiles[i]);
    				fileSystem.root.getFile("Logs/"+listOfLogFiles[i], {create: true}, gotTraceLogFileEntry, failTraceLogFileEntry);
				}
		}

	} 
	
}

/**
 * Failure Callback Function for performanceLogFileReceived.createWriter
 * 
 * @param {} error
 */
function failPerformanceLogFileWriter(error)
{
   //console.log("Performance Log file writer could not be created. Still launching the app. Error is " + error.code);
   if(!app)
   		appCreate(); 

}

/**
 * Failure CallBack Function for createPerformanceLogFile
 * @param {} error - Error in creating Performance Log file.
 */
function failPerformanceLogFileEntry(error)
{
	//console.log("Performance Log file could not be created. Still launching the app. Error is"+error.code);	
	if(!app)
		appCreate(); 

}

/**
 * Function for adding the Log to a pool
 * @param {} log - Log to be stored in temp pool
 */
function addLogToPerformanceLogPool(performanceLog)
{	

	//Push the newest Log Entry to the log pool
	performanceLogsPool.push(performanceLog); 
		
	//Process the performance Log pool only if it is not being already processed and the file writer is also created. 
	if(!boolPerformanceLogProcessing && performanceLogFileWriter)
		processPerformanceLogPool("forced");
}

/**
 * Function to process the Performance Log Pool
 * @param {} string
 */
function processPerformanceLogPool(string)
{	
	boolPerformanceLogProcessing = true;
    
	if(performanceLogFileWriter)
    	{    	
    		//Fetching the oldest entry in the pool
    		
    		if(performanceLogsPool.length>0)
    		{
    			performanceLog = performanceLogsPool.shift();	
    			
	   			if(performanceLog)
	    			{
	   					performanceLogFileWriter.seek(performanceLogFileWriter.length);
		
	   					performanceLogFileWriter.write("\n"+performanceLog.date+" "+performanceLog.time+"  "+performanceLog.details+" "+performanceLog.message);
	    			}
    		}
    	
    		else
    		{
    			boolPerformanceLogProcessing = false;
    		}
   	}
}

