//  2012 SAP AG. All rights reserved.
// Author : SAP.

/**
 * 
 * Global Variable Declaration 
 */

var traceLogFile;
var traceLogFileEntry;
var traceLogFileWriter;
var traceLog;
var traceLogsPool=[ ];
var boolTraceLogProcessing = false;

/**
 * 	Function to set the Log level for the Client Side to any of the following
 * 
 * 	jQuery.sap.log.Level.FATAL Fatal level(0).
 *	jQuery.sap.log.Level.ERROR Error level(1).
 *	jQuery.sap.log.Level.WARNING Warning level(2).	
 *	jQuery.sap.log.Level.INFO Info level(3).
 *	jQuery.sap.log.Level.DEBUG Debug level(4).
 *	jQuery.sap.log.Level.TRACE Trace level(5).
 * 	jQuery.sap.log.Level.NONE Do not log anything
 *
 *  Higher numeric values represent less important levels
 * 
 *  Setting it to jQuery.sap.log.Level.DEBUG for development purposes
 * 
 * 	@param {jQuery.sap.log.Level} logLevel
 */


function setTraceLogLevel(logLevel) 
{
	
	if(logLevel == "ON")
		logLevel = jQuery.sap.log.Level.INFO;
		
	else if(logLevel == "OFF")
		logLevel = jQuery.sap.log.Level.NONE;
    			
	//Set Log Level for Trace Log

	jQuery.sap.log.setLevel(logLevel, "Trace");

}

/**
 * Creates a new info-level entry in the log with the given message, details and calling component.
 *
 * @param {} message Message text to display
 * @param {} detail Details about the message, might be omitted
 * @param {} component Name of the component that produced the log entry
 */
function logTraceInfo(message,detail)
{
	var oldLogCount = jQuery.sap.log.getLogEntries().length;

	jQuery.sap.log.info(message,detail,"Trace");
	
	var newLogCount = jQuery.sap.log.getLogEntries().length;
	
	if(oldLogCount!=newLogCount)
	{
		//Check if Trace log file is created. Log the statements only when file is created. 
		if(traceLogFileEntry)
		{
			addLogToTraceLogPool(jQuery.sap.log.getLogEntries()[jQuery.sap.log.getLogEntries().length-1]);
		}
	}	
}


/**
 * Success CallBack Function for createTraceLogFile
 * 
 * @param {} traceFile - Techincal Log File created
 * 
 */
function gotTraceLogFileEntry(traceLogFileEntryReceived)
{
	//console.log("Trace Log File Entry created at"+traceLogFileEntryReceived.fullPath);
	traceLogFileEntry = traceLogFileEntryReceived;
	traceLogFileEntry.file(gotTraceLogFile,failTraceLogFile);
}

function gotTraceLogFile(traceLogFileReceived)
{
	//Creating the TraceLogFile Writer
	traceLogFile = traceLogFileReceived;
	traceLogFileEntry.createWriter(gotTraceLogFileWriter, failTraceLogFileWriter);	   
}

function failTraceLogFile(error)
{
   	//console.log("Failed to create Trace log file. Error is "+error.code+"Launching the app now");
  	
  	if(!app)
			appCreate();
}

/**
 * Success Callback Function for traceLogFileReceived.createWriter
 * 
 * @param {} traceLogFileWriterReceived - The received TraceLogFile writer
 */

function gotTraceLogFileWriter(traceLogFileWriterReceived)
{
    //console.log("Trace Log File Writer Created");
   
    traceLogFileWriter = traceLogFileWriterReceived;
    
    traceLogFileWriter.onwritestart = function(evt)  
	  	{
	  		boolTraceLogProcessing = true;
	};
	  
traceLogFileWriter.onwriteend = function(evt) 
	  	{
		processTraceLogPool("callback");
	};     		
	  
traceLogFileWriter.onerror = function(evt)
	  	{
	        //console.log("Trace Log File. Write to file couldnot be performed"+evt.code);
	  	}
   
    //console.log("Setting the UserLog file level to INFO");

    setUserLogLevel("ON");
    
    if (device.platform.match("Android"))
	{
    	//console.log("Creating User Log file for Android "+"SAPLoyaltyCustomerLinkUserLog");
    	fileSystem.root.getFile("SAPCustomerLoyaltyActivity/SAPLoyaltyCustomerLinkUserLog.txt", {create: true}, gotUserLogFileEntry, failUserLogFileEntry);

	}
	
	else
	{
		//console.log("Creating User Log file for iOS "+"SAPLoyaltyCustomerLinkUserLog");
		fileSystem.root.getFile("Logs/SAPLoyaltyCustomerLinkUserLog.txt", {create: true}, gotUserLogFileEntry, failUserLogFileEntry);
	}
	
   	
     											  

}

/**
 * Failure Callback Function for traceLogFileReceived.createWriter
 * 
 * @param {} error
 */
function failTraceLogFileWriter(error)
{
   //console.log("Trace Log file writer could not be created. Still launching the app. Error is " + error.code);
   if(!app)
   		appCreate(); 
}

/**
 * Failure CallBack Function for createTraceLogFile
 * @param {} error - Error in creating Trace Log file.
 */
function failTraceLogFileEntry(error)
{
	//console.log("Trace Log file could not be created. Still launching the app. Error is"+error.code);	
	if(!app)
		appCreate(); 
}

/**
 * Function for adding the Log to a pool
 * @param {} log - Log to be stored in temp pool
 */
function addLogToTraceLogPool(traceLog)
{
	//Push the newest Log Entry to the log pool
	traceLogsPool.push(traceLog); 
		
	//Process the trace Log pool only if it is not being already processed and the file writer is also created. 
	if(!boolTraceLogProcessing && traceLogFileWriter)
		processTraceLogPool("forced");
}

/**
 * Function to process the Trace Log Pool
 * @param {} string
 */
function processTraceLogPool(string)
{	
    boolTraceLogProcessing = true;
    
	if(traceLogFileWriter)
    	{    	
    		//Fetching the oldest entry in the pool
    		
    		if(traceLogsPool.length>0)
    		{
    			traceLog = traceLogsPool.shift();	
    			
   			if(traceLog)
    			{
					traceLogFileWriter.seek(traceLogFileWriter.length);
		
					traceLogFileWriter.write
									( "\n"+traceLog.date+" "+traceLog.time+"  "+traceLog.details+" "+traceLog.message);
    			}
    		}
    	
    		else
    		{
    			boolTraceLogProcessing = false;
    		}
   	}
}

